'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class BannerService extends Service {
  async create({ title, path, annex, content, annexname, date }) {
    assert(title, '标题不存在');
    assert(path, '轮播图不存在');
    assert(content, '内容不存在');
    const { Banner: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      await model.create({ title, date, slug, path, annex, content, createAt, annexname });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      console.log(error);
      throw new Error({ errcode: -2001, errmsg: '添加失败' });
    }
  }
  async update({ title, path, annex, content, date, _id, annexname }) {
    assert(_id, 'id不存在');
    const { Banner: model } = this.ctx.model;
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      await model.findById(_id).update({ title, slug, path, annex, date, content, annexname });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '修改失败' });
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Banner: model } = this.ctx.model;
    try {
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async details({ id }) {
    assert(id, 'id不存在');
    const { Banner: model } = this.ctx.model;
    try {
      const res = await model.findById(id);
      return { errmsg: '', errcode: 0, data: res };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '查询失败' });
    }
  }
  async query({ skip, limit, title }) {
    const { Banner: model } = this.ctx.model;
    const filter = {};
    if (title) filter.title = title;
    const total = await model.find({ ...filter });
    try {
      let res;
      if (skip && limit) {
        res = await model.find({ ...filter }, { content: false }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter }, { content: false });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error('查询失败');
    }
  }
}

module.exports = BannerService;
