'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class LinkService extends Service {
  async create({ name, path, links }) {
    assert(name, '名称不存在');
    assert(path, '路径不存在');
    assert(links, '链接不存在');
    const { Links: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      await model.create({ name, path, links, createAt });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('添加失败');
    }
  }
  async update({ name, path, _id, links }) {
    assert(_id, 'id不存在');
    const { Links: model } = this.ctx.model;
    try {
      await model.findById(_id).update({ name, path, links });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('修改失败');
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Links: model } = this.ctx.model;
    try {
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('删除失败');
    }
  }
  async query({ skip, limit, name }) {
    const { Links: model } = this.ctx.model;
    try {
      const filter = {};
      if (name) filter.name = name;
      let res;
      const total = await model.find({ ...filter });
      if (skip && limit) {
        res = await model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error('查询失败');
    }
  }
}

module.exports = LinkService;
