'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class MenuService extends Service {
  async create({ name, code, state, type, uri, en, sort, pages, column }) {
    assert(name, '名称不存在');
    assert(code, '编码不存在');
    assert(state, '状态不存在');
    assert(type, '类型不存在');
    assert(en, '英文缩写不存在');
    assert(sort, '排序不存在');
    const { Menu: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      const isdata = await model.find({ code });
      if (isdata.length > 0) return { errmsg: '编码已存在', errcode: -2001 };
      await model.create({ name, code, createAt, state, type, uri, en, sort: Number(sort), pages, column });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('添加失败');
    }
  }
  async update({ name, state, _id, uri, en, sort, pages, column }) {
    assert(_id, 'id不存在');
    const { Menu: model } = this.ctx.model;
    try {
      await model.findById(_id).update({ name, state, uri, en, sort: Number(sort), pages, column });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('修改失败');
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Menu: model } = this.ctx.model;
    const { Role: Rolemodel } = this.ctx.model;
    const { Content: Contentmodel } = this.ctx.model;
    try {
      const menu = await model.findById(id);
      const roleList = await Rolemodel.where('adminMenuList').in(menu.code);
      roleList.forEach(async e => {
        const adminMenuList = e.adminMenuList.filter(p => p !== menu.code);
        await Rolemodel.findById(e._id).update({ adminMenuList });
      });
      switch (menu.type) {
        case '0': {
          // 栏目
          const list = await Contentmodel.find({ menus: menu.code });
          if (list.length > 0) {
            return { errmsg: '存在绑定文章，请先删除文章', errcode: -1001 };
          }
          break;
        }
        default: {
          break;
        }
      }
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('删除失败');
    }
  }
  // 新增排序
  async query({ skip, limit, name, code, type }) {
    const { Menu: model } = this.ctx.model;
    try {
      const filter = {};
      if (name) filter.name = name;
      if (code) filter.code = code;
      if (type) filter.type = type;
      const total = await model.find({ ...filter });
      let res;
      if (skip !== undefined && limit !== undefined) {
        res = await model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit))
          .sort({ sort: 1 });
      } else {
        res = await model.find({ ...filter }).sort({ sort: 1 });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error('查询失败');
    }
  }
}

module.exports = MenuService;
