'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class PageService extends Service {
  async create({ title, annex, content, istop, annexname, date, code }) {
    assert(title, '标题不存在');
    assert(content, '内容不存在');
    assert(code, '编码不存在');
    const { Page: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      await model.create({ title, slug, date, annex, content, istop, createAt, annexname, code });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('添加失败');
    }
  }
  async update({ title, annex, content, istop, _id, annexname, date, code }) {
    assert(_id, 'id不存在');
    const { Page: model } = this.ctx.model;
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      await model.findById(_id).update({ title, date, slug, annex, content, istop, annexname, code });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('修改失败');
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Page: model } = this.ctx.model;
    const { Menu: Menumodel } = this.ctx.model;
    try {
      const menulist = await Menumodel.find({ menus: id });
      menulist.filter(async p => {
        p.menus = '';
        await Menumodel.findById(p._id).update(p);
      });
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error('删除失败');
    }
  }
  async details({ id }) {
    assert(id, 'id不存在');
    const { Page: model } = this.ctx.model;
    try {
      const res = await model.findById(id);
      return { errmsg: '', errcode: 0, data: res };
    } catch (error) {
      throw new Error('删除失败');
    }
  }
  async detailscode({ code }) {
    assert(code, 'code不存在');
    const { Page: model } = this.ctx.model;
    try {
      const res = await model.findOne({ code });
      return { errmsg: '', errcode: 0, data: res };
    } catch (error) {
      throw new Error('删除失败');
    }
  }
  async query({ skip, limit, title }) {
    const { Page: model } = this.ctx.model;
    const filter = {};
    if (title) filter.title = title;
    try {
      const total = await model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await model.find({ ...filter }, { content: false }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter }, { content: false });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error('查询失败');
    }
  }
}

module.exports = PageService;
